import boto3
import os
import json
from exception import *

DYNAMO_CLIENT = boto3.client('dynamodb')
LAMBDA_CLIENT = boto3.client('lambda')


def lambda_handler(event, context):
    """

    :param event:
        {
            "object_id": String
            "attribute_name": String,
            "attribute_value": String,
            "level" (optional): Array,
            "object_type_id" (optional): String
        }
    :param context:
    :return:
    """

    if 'object_id' not in event:
        raise InvalidParameterException('You need an object_id to invoke this function')
    if 'attribute_name' not in event or 'attribute_value' not in event:
        raise InvalidParameterException('You need to specify an attribute name and value')
    if 'level' in event and 'object_type_id' in event:
        raise InvalidParameterException('You cannot have both a level and an object_type_id in the payload')

    object_id = event['object_id']

    if 'level' in event:
        nodes = get_children(object_id, levels=event['level'])
    elif 'object_type_id' in event:
        nodes = get_children(object_id, type_id=event['object_type_id'])
    else:
        raise InvalidParameterException('You need to specify a level or an object_type_id to post to')

    post_to_mqtt(nodes, event['attribute_name'], event['attribute_value'])

    return "SUCCESS"


def get_children(object_id, levels=[], result_list=[], level=1, type_id=None):
    if len(levels) > 0 and level == max(levels) + 1:
        return

    objects = DYNAMO_CLIENT.query(
        TableName='fndy_object_reference_' + os.environ['ENV'],
        KeyConditionExpression='refTo = :val1',
        ExpressionAttributeValues={
            ':val1': {
                'S': object_id
            }
        }
    )

    if objects['Count'] == 0:
        return result_list

    for object in objects['Items']:
        child_id = object['refFrom']['S']
        child_describe = DYNAMO_CLIENT.get_item(
            TableName='fndy_object_info_' + os.environ['ENV'],
            Key={
                'deviceId': {
                    'S': child_id
                }
            }
        )
        mqtt_topic = child_describe['Item']['a.mqtt_topic']['S']
        obj_type_id = child_describe['Item']['typeId']['S']

        if type_id is not None and type_id == obj_type_id:
            result_list.append(mqtt_topic)
        elif level in levels:
            result_list.append(mqtt_topic)
        get_children(child_id, levels, result_list, level + 1, type_id)

    return result_list


def post_to_mqtt(topics, attribute_name, attribute_val):
    payload = {
        attribute_name: attribute_val
    }
    for topic in topics:
        mqtt_payload = {
            "body": {
                "topic": topic,
                "payload": json.dumps(payload)
            },
            "invoke": True
        }

        LAMBDA_CLIENT.invoke(
            FunctionName="fndy-post-publish-mqtt-secure-" + os.environ['ENV'],
            InvocationType="RequestResponse",
            Payload=json.dumps(mqtt_payload)
        )

    return 'SUCCESS'
